var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Execution: () => Execution,
  ProviderClass: () => ProviderClass,
  Scope: () => Scope,
  combine: () => combine,
  create: () => create,
  createExecution: () => createExecution,
  createScope: () => createScope,
  dispose: () => dispose,
  execute: () => execute,
  getScope: () => getScope,
  group: () => group,
  isExecutor: () => isExecutor,
  prepare: () => prepare,
  resolve: () => resolve,
  resolveValue: () => resolveValue,
  scoper: () => scoper,
  value: () => value
});
module.exports = __toCommonJS(src_exports);
var ProviderClass = class {
  constructor(provider) {
    this.provider = provider;
  }
};
var Scope = class {
  constructor(store = /* @__PURE__ */ new Map(), resolves = [], defers = []) {
    this.store = store;
    this.resolves = resolves;
    this.defers = defers;
  }
  has(executor) {
    return this.store.has(executor);
  }
  async get(executor) {
    return this.store.get(executor);
  }
  set(executor, v) {
    if (this.store.has(executor)) {
      return;
    }
    if (isExecutor(v)) {
      this.store.set(executor, v.resolve(this));
      return;
    }
    this.store.set(executor, Promise.resolve(v));
  }
  resolveValue(executor, value2) {
    if (this.store.has(executor)) {
      return;
    }
    if (isExecutor(value2)) {
      this.store.set(executor, value2.resolve(this));
      return;
    }
    this.store.set(executor, Promise.resolve(value2));
  }
  async resolve(executor) {
    return isExecutor(executor) ? executor.resolve(this) : combine(executor).resolve(this);
  }
  async execute(executable, dependency) {
    const value2 = await this.resolve(dependency);
    return await executable(value2);
  }
  prepare(provider, dependency) {
    const scope = this;
    return async function() {
      const that = this;
      const args = arguments;
      return execute(async (d) => provider.call(that, d, ...args), dependency, scope);
    };
  }
  addDefer(deferer) {
    this.defers.push(deferer);
  }
  addOnResolves(onResolve) {
    this.resolves.push(onResolve);
  }
  get onResolves() {
    return this.resolves;
  }
  async dispose() {
    this.defers.forEach((d) => d());
    this.store.clear();
    this.defers = [];
    this.resolves = [];
  }
};
function createScope() {
  return new Scope();
}
function getScope() {
  return globalScope;
}
var globalScope = createScope();
function dispose() {
  globalScope.dispose();
}
var executorSymbol = Symbol.for("$submodule");
function isExecutor(obj) {
  return obj?.[executorSymbol];
}
var Execution = class {
  constructor(executor, dependency) {
    this.executor = executor;
    this.dependency = dependency;
  }
  async executeIn(scope, ...input) {
    const dependency = await scope.resolve(this.dependency);
    return await this.executor(dependency, ...input);
  }
  async execute(...input) {
    return await this.executeIn(getScope(), ...input);
  }
  resolve(scope = getScope()) {
    return this.executeIn.bind(this, scope);
  }
};
function createExecution(executor, dependency) {
  return new Execution(executor, dependency);
}
function create(provider, dependencies) {
  let modifiableDependency = dependencies ? isExecutor(dependencies) ? dependencies : combine(dependencies) : void 0;
  let patches = [];
  async function resolve2(scope) {
    if (scope.has(executor)) {
      return await scope.get(executor);
    }
    const promise = Promise.resolve().then(() => {
      for (const [executor2, patch2] of patches) {
        scope.set(executor2, patch2);
      }
    }).then(() => modifiableDependency?.resolve(scope)).then(async (actualized) => {
      if (provider instanceof ProviderClass) {
        return provider.provider(scope, actualized);
      }
      if (provider.length > 0 && actualized === void 0) {
        throw new Error(`invalid state, provider ${provider.toString()} requires dependent but not provided`);
      }
      return provider(actualized);
    }).then((actualized) => {
      for (const onResolve of scope.onResolves) {
        if (onResolve.filter(actualized)) {
          return onResolve.cb(actualized);
        }
      }
      return actualized;
    }).catch((e) => {
      throw e;
    });
    scope.set(executor, promise);
    return await promise;
  }
  function patch(executor2, patch2) {
    patches.push([executor2, patch2]);
  }
  function reset() {
    patches = [];
  }
  const executor = { reset, resolve: resolve2, patch, [Symbol.for("$submodule")]: true };
  return executor;
}
var value = (value2) => create(() => value2);
var group = (...values) => create(
  new ProviderClass(
    async (scope) => Promise.all(values.map((v) => scope.resolve(v)))
  )
);
var scoper = create(new ProviderClass(async (scope) => scope));
function prepare(provider, dependency, scope = getScope()) {
  const execution = new Execution(provider, dependency);
  return execution.resolve(scope);
}
async function execute(executable, dependency, scope = getScope()) {
  const execution = new Execution(executable, dependency);
  return execution.executeIn(scope);
}
async function resolve(executor, scope = getScope()) {
  return await scope.resolve(executor);
}
function resolveValue(executor, value2, scope = getScope()) {
  scope.resolveValue(executor, value2);
}
function combine(layout) {
  return create(new ProviderClass(async (scope) => {
    const layoutPromise = Object.entries(layout).map(([key, executor]) => executor.resolve(scope).then((value2) => [key, value2]));
    const result = Object.fromEntries(await Promise.all(layoutPromise));
    return result;
  }));
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Execution,
  ProviderClass,
  Scope,
  combine,
  create,
  createExecution,
  createScope,
  dispose,
  execute,
  getScope,
  group,
  isExecutor,
  prepare,
  resolve,
  resolveValue,
  scoper,
  value
});
//# sourceMappingURL=index.js.map